@@ -2,18 +2,10 @@
 # Author: Simeon Reusch (simeon.reusch@desy.de)
 # License: BSD-3-Clause
 
-import multiprocessing, time, os, sys, logging, argparse, tarfile
+import multiprocessing, time, os, sys, logging, argparse, tarfile, shutil, warnings
 
 from tqdm import tqdm
-
-from ztflc import forcephotometry
-from ztflc.io import LOCALDATA
-
-import ztfquery
-from ztfquery import query as zq
-
 import numpy as np
-
 import pandas as pd
 
 from astropy.time import Time
@@ -22,6 +14,11 @@ from astropy import units as u
 from astropy.utils.console import ProgressBar
 import requests.exceptions
 
+from ztflc import forcephotometry
+from ztflc.io import LOCALDATA
+import ztfquery
+from ztfquery import query as zq
+
 from fpbot import database, credentials
 from fpbot.thumbnails import generate_thumbnails
 from fpbot.utils import (
@@ -65,8 +62,6 @@ for path in [
     if not os.path.exists(path):
         os.makedirs(path)
 
-logger = logging.getLogger(__name__)
-
 
 class ForcedPhotometryPipeline:
     """ """
@@ -88,37 +83,39 @@ class ForcedPhotometryPipeline:
         sciimg=False,
         update_enforce=False,
         update_disable=False,
-        ampel=False,
+        ampel=True,
         download_newest=True,
         filecheck=False,
         verbose=False,
     ):
 
-        self.startime = time.time()
+        self.logger = logging.getLogger(__name__)
+        self.logger.setLevel(logging.INFO)
 
         # check for IRSA credentials
         _, _ = credentials.get_user_and_password("irsa")
 
+        self.startime = time.time()
+
         if file_or_name is None:
-            logger.error(
+            self.logger.error(
                 "You have to initialize this class with at least one name of a ZTF object for which to perform forced photometry a textfile containing one ZTF name per line or an arbitrary name if the -radec option is chosen."
             )
         else:
             self.file_or_name = file_or_name
 
-        if not logger.hasHandlers():
+        if not self.logger.hasHandlers():
             logFormatter = logging.Formatter(
                 "%(asctime)s [%(threadName)-12.12s] [%(levelname)-5.5s]  %(message)s"
             )
 
             fileHandler = logging.FileHandler("./log")
             fileHandler.setFormatter(logFormatter)
-            logger.addHandler(fileHandler)
+            self.logger.addHandler(fileHandler)
 
             consoleHandler = logging.StreamHandler()
             consoleHandler.setFormatter(logFormatter)
-            logger.addHandler(consoleHandler)
-            logger.setLevel(logging.INFO)
+            self.logger.addHandler(consoleHandler)
 
         self.daysago = daysago
         self.daysuntil = daysuntil
@@ -148,10 +145,8 @@ class ForcedPhotometryPipeline:
                 coords = SkyCoord(f"{ra} {dec}", unit=(u.hourangle, u.deg))
             else:
                 coords = SkyCoord(f"{ra} {dec}", unit=u.deg)
-            self.ra = np.float(
-                coords.ra.to_string(decimal=True, unit=u.deg, precision=8)
-            )
-            self.dec = np.float(
+            self.ra = float(coords.ra.to_string(decimal=True, unit=u.deg, precision=8))
+            self.dec = float(
                 coords.dec.to_string(decimal=True, unit=u.deg, precision=8)
             )
             if isinstance(self.file_or_name, list):
@@ -162,7 +157,7 @@ class ForcedPhotometryPipeline:
             self.update_database_with_given_radec()
 
         elif (ra is None and dec is not None) or (ra is not None and dec is None):
-            logger.info("Either both set RA and Dec or none.")
+            self.logger.info("Either both set RA and Dec or none.")
             raise ValueError
 
         else:
@@ -241,22 +236,22 @@ class ForcedPhotometryPipeline:
                     elif is_wise_name(line):
                         self.object_list.append(line)
             except FileNotFoundError as error:
-                logger.error(errormessage)
+                self.logger.error(errormessage)
                 raise error
             assert (
                 self.object_list[0][:3] == "ZTF" or self.object_list[0][:4] == "WISE"
             ), errormessage
         # Grammar check
         if len(self.object_list) == 1:
-            logger.info(
+            self.logger.info(
                 f"Doing forced photometry for {len(self.object_list)} transient"
             )
         else:
-            logger.info(
+            self.logger.info(
                 f"Doing forced photometry for {len(self.object_list)} transients"
             )
 
-        logger.info("Logs are stored in log")
+        self.logger.info("Logs are stored in log")
 
     def purge(self):
         """Deletes transient(s) from the database
@@ -264,12 +259,12 @@ class ForcedPhotometryPipeline:
         """
         from fpbot.utils import get_local_files
 
-        logger.info(f"Will delete (from disk and db): {self.object_list}")
+        self.logger.info(f"Will delete (from disk and db): {self.object_list}")
 
         for name in tqdm(self.object_list):
             local_files = get_local_files(names=[name])
 
-            logger.info(f"Deleting {len(local_files)} local files for {name}")
+            self.logger.info(f"Deleting {len(local_files)} local files for {name}")
 
             if local_files:
                 for file in local_files:
@@ -278,7 +273,7 @@ class ForcedPhotometryPipeline:
                     if os.path.exists(file + ".md5"):
                         os.remove(file + ".md5")
 
-            logger.info(f"Deleting {name} from internal database")
+            self.logger.info(f"Deleting {name} from internal database")
             database.delete_from_database(name)
 
     def check_for_duplicates(self):
@@ -310,12 +305,12 @@ class ForcedPhotometryPipeline:
         """
         Check for entry in Mongo database and update it via AMPEL
         """
-        logger.info("\nChecking database.")
+        self.logger.info("Checking database.")
 
         needs_external_database = []
 
         if self.update_enforce:
-            logger.info("\nForced update of alert data data from AMPEL")
+            self.logger.info("Forced update of alert data data from AMPEL")
 
         query = database.read_database(self.object_list, ["_id", "entries", "ra"])
 
@@ -343,42 +338,40 @@ class ForcedPhotometryPipeline:
             ):
                 needs_external_database.append(name)
 
-        logger.info("\nConnecting to AMPEL.")
+        self.logger.info("Connecting to AMPEL.")
 
         from fpbot import connectors
 
         try:
-            connector = connectors.AmpelInfo(
-                ztf_names=needs_external_database, logger=logger
-            )
+            connector = connectors.AmpelInfo(ztf_names=needs_external_database)
             ampel_failed = False
         except:
             ampel_failed = True
 
         if ampel_failed:
-            logger.error(
-                "\nConnection to AMPEL failed. Please check if the AMPEL API is currently working."
+            self.logger.error(
+                "Connection to AMPEL failed. Please check if the AMPEL API is currently working."
             )
 
         if self.jdmin is None:
             if self.daysago is None:
-                logger.info(
-                    "\nNo 'daysago' given, full timerange since ZTF operations used."
+                self.logger.info(
+                    "No 'daysago' given, full timerange since ZTF operations used."
                 )
             else:
                 if self.daysuntil is None:
-                    logger.info(
-                        f"\nData from {self.daysago:.2f} days ago till today is used."
+                    self.logger.info(
+                        f"Data from {self.daysago:.2f} days ago till today is used."
                     )
                 else:
-                    logger.info(
-                        f"\nData from {self.daysago:.2f} days ago till {self.daysuntil:.2f} days ago is used."
+                    self.logger.info(
+                        f"Data from {self.daysago:.2f} days ago till {self.daysuntil:.2f} days ago is used."
                     )
 
         now = Time(time.time(), format="unix", scale="utc").jd
 
         if not ampel_failed:
-            logger.info("\nUpdating local database.")
+            self.logger.info("Updating local database.")
 
             for index, result in enumerate(tqdm(connector.queryresult)):
                 if result is not None:
@@ -410,7 +403,7 @@ class ForcedPhotometryPipeline:
         Delete from object-list if no info is available
         """
 
-        logger.info("\nChecking if alert data is present in the local database.")
+        self.logger.info("Checking if alert data is present in the local database.")
 
         query = database.read_database(self.object_list, ["name", "entries"])
         not_found = []
@@ -424,8 +417,8 @@ class ForcedPhotometryPipeline:
         if not_found:
             for index in sorted(del_indices, reverse=True):
                 del self.object_list[index]
-            logger.info(
-                f"\nThese could not be found in meta database. Will not be downloaded or fit: {not_found}"
+            self.logger.info(
+                f"These could not be found in the local database. Will not be downloaded or fit: {not_found}"
             )
 
     def download(self):
@@ -458,8 +451,6 @@ class ForcedPhotometryPipeline:
 
         from fpbot.connectors import get_ipac_and_local_filecount
 
-        logger.info(f"\nObtaining information on available images at IRSA.")
-
         ipac_filecounts = get_ipac_and_local_filecount(
             ztf_names=download_requested,
             ras=ras,
@@ -474,11 +465,11 @@ class ForcedPhotometryPipeline:
                 download_needed.append(name)
 
         if download_needed:
-            logger.info(
-                f"\n{len(download_needed)} of {len(self.object_list)} objects have additional images available at IRSA.\nThese will be downloaded now."
+            self.logger.info(
+                f"{len(download_needed)} of {len(self.object_list)} objects have additional images available at IRSA.\nThese will be downloaded now."
             )
         else:
-            logger.info(
+            self.logger.info(
                 "For none of the transients are new images available, so no download needed."
             )
 
@@ -499,12 +490,22 @@ class ForcedPhotometryPipeline:
                 name=name,
             )
 
-            logger.info(f"\n{name} ({i+1} of {len(download_needed)}) Downloading data.")
-            if not os.path.exists(
-                os.path.join(MARSHALDATA, "Cosmology_target_sources.csv")
-            ):
-                fp.io.update_marshal()
+            self.logger.info(
+                f"{name} ({i+1} of {len(download_needed)}) Downloading data."
+            )
+
+            marshal_dummyfile = os.path.join(
+                os.path.dirname(os.path.realpath(__file__)),
+                "data",
+                "marshal_dummyfile.csv",
+            )
+            dummyfile_target = os.path.join(MARSHALDATA, "Cosmology_target_sources.csv")
+
+            if not os.path.exists(dummyfile_target):
+                shutil.copyfile(marshal_dummyfile, dummyfile_target)
+
             fp.load_metadata()
+
             if self.sciimg:
                 fp.io.download_data(
                     nprocess=32,
@@ -621,12 +622,27 @@ class ForcedPhotometryPipeline:
                 jdmax=jdmax,
                 name=name,
             )
-            logger.info(f"\n{name} ({i+1} of {objects_total}) loading metadata.")
+
+            marshal_dummyfile = os.path.join(
+                os.path.dirname(os.path.realpath(__file__)),
+                "data",
+                "marshal_dummyfile.csv",
+            )
+            dummyfile_target = os.path.join(MARSHALDATA, "Cosmology_target_sources.csv")
+
+            if not os.path.exists(dummyfile_target):
+                shutil.copyfile(marshal_dummyfile, dummyfile_target)
+
+            self.logger.info(f"{name} ({i+1} of {objects_total}) loading metadata.")
             fp.load_metadata()
-            logger.info(f"\n{name} ({i+1} of {objects_total}) metadata loaded.")
-            logger.info(f"\n{name} ({i+1} of {objects_total}) loading paths to files.")
+            self.logger.info(f"{name} ({i+1} of {objects_total}) metadata loaded.")
+            self.logger.info(
+                f"{name} ({i+1} of {objects_total}) loading paths to files."
+            )
             fp.load_filepathes(filecheck=self.filecheck)
-            logger.info(f"\n{name} ({i+1} of {objects_total}) paths to files loaded.")
+            self.logger.info(
+                f"{name} ({i+1} of {objects_total}) paths to files loaded."
+            )
 
             """
             Check how many forced photometry datapoints
@@ -638,17 +654,24 @@ class ForcedPhotometryPipeline:
                 fitted_datapoints = 0
 
             df_file = os.path.join(FORCEPHOTODATA, f"{name}.csv")
+
             if os.path.isfile(df_file):
-                _df = pd.read_csv(df_file, comment="#", index_col=0)
-                if len(_df) == 0:
+                if os.stat(df_file).st_size > 1:
+                    _df = pd.read_csv(df_file, comment="#", index_col=0)
+                    if len(_df) == 0:
+                        force_refit = True
+                else:
+                    os.remove(df_file)
                     force_refit = True
             else:
                 force_refit = True
 
             # Compare to number of fitted datapoints from database
             if number_of_fitted_datapoints_expected > fitted_datapoints or force_refit:
-                logger.info(f"\n{name} ({i+1} of {objects_total}): Fitting PSF.")
+                self.logger.info(f"{name} ({i+1} of {objects_total}): Fitting PSF.")
 
+                # with warnings.catch_warnings():
+                #     warnings.simplefilter("ignore")
                 fp.run_forcefit(
                     verbose=self.verbose,
                     nprocess=nprocess,
@@ -688,8 +711,8 @@ class ForcedPhotometryPipeline:
                     },
                 )
             else:
-                logger.info(
-                    f"\n{name} ({i+1} of {objects_total}) No new images to fit, skipping PSF fit."
+                self.logger.info(
+                    f"{name} ({i+1} of {objects_total}) No new images to fit, skipping PSF fit."
                 )
 
     def plot(
@@ -699,7 +722,7 @@ class ForcedPhotometryPipeline:
         Plots the lightcurve (uses PSF fitted datapoints if available and
         checks for alert photometry otherwise)
         """
-        logger.info(f"\nPlotting")
+        self.logger.info(f"Plotting")
         object_count = len(self.object_list)
         if snt == None:
             snt = [self.snt] * object_count
@@ -741,13 +764,13 @@ class ForcedPhotometryPipeline:
         """
         Check if each image downloaded from IPAC with ztfquery can be opened
         """
-        logger.info(
+        self.logger.info(
             "Running filecheck. This can take several hours, depending on the size of your $ZTDFATA folder."
         )
         badfiles = ztfquery.io.run_full_filecheck(
             erasebad=True, nprocess=self.nprocess, redownload=True
         )
-        logger.info(f"BADFILES:\n{badfiles}")
+        self.logger.info(f"BADFILES:\n{badfiles}")
 
     @staticmethod
     def _plot_multiprocessing_(args):
@@ -776,7 +799,6 @@ class ForcedPhotometryPipeline:
             plot_flux=plot_flux,
             plot_alertdata=plot_alertdata,
         )
-        print(f"\n{name} plotted")
 
     def saltfit(self, snt=5, quality_checks=False, progress=True, alertfit=False):
         """
@@ -793,8 +815,8 @@ class ForcedPhotometryPipeline:
 
         objectcount = len(self.cleaned_object_list)
 
-        logger.info(
-            "\nChecking if the mwebv Milky Way dust map value is present and compute it if not."
+        self.logger.info(
+            "Checking if the mwebv Milky Way dust map value is present and compute it if not."
         )
 
         for index, name in enumerate(tqdm(self.cleaned_object_list)):
@@ -845,9 +867,9 @@ class ForcedPhotometryPipeline:
 
         for index, name in enumerate(tqdm(self.cleaned_object_list)):
             if alertfit:
-                logger.info(f"\n{name} performing SALT fit for alert photometry.")
+                self.logger.info(f"{name} performing SALT fit for alert photometry.")
             else:
-                logger.info(f"\n{name} performing SALT fit for forced photometry.")
+                self.logger.info(f"{name} performing SALT fit for forced photometry.")
 
             try:
                 fitresult, fitted_model = fit_salt(
@@ -860,7 +882,7 @@ class ForcedPhotometryPipeline:
                 fitresults.append(fitresult)
                 fitted_models.append(fitted_model)
             except:
-                logger.info(f"\n{name} Error while fitting.")
+                self.logger.info(f"{name} Error while fitting.")
 
         for fitresult in fitresults:
             if fitresult is not None:
@@ -885,13 +907,13 @@ class ForcedPhotometryPipeline:
 
         salt_dataframe.to_csv(salt_dataframe_path)
 
-        logger.info(
-            f"\n{len(fitresult_df)} of {object_count} fits were performed successfully.\n"
+        self.logger.info(
+            f"{len(fitresult_df)} of {object_count} fits were performed successfully."
         )
 
     def sendmail(self, send_to, tarball=False):
         """ """
-        logger.info("\nSending mail.")
+        self.logger.info("Sending mail.")
         import smtplib
 
         from email.mime.application import MIMEApplication
